@@ -85,12 +85,12 @@ ctmaOptimizeInit <- function(primaryStudies=NULL,
   newStudyList <- as.list(listElements)
   validElements <- c("deltas", "sampleSizes", "pairwiseNs", "empcovs", "moderators", "startValues", "studyNumbers", "rawData", "empMeans", "empVars",
                      "source", "ageM", "malePercent", "occupation", "country", "alphas", "targetVariables", "recodeVariables", "combineVariables",
-                     "combineVariablesNames", "missingVariables") #, "n.studies", "summary", "excelSheets", "plot.type")
+                     "combineVariablesNames", "missingVariables", "inits", "emprawList") #, "n.studies", "summary", "excelSheets", "plot.type")
   counter <- 0
   for (i in listElements) {
     counter <- counter + 1
     if (i %in% validElements) {
-      if (i %in% c("pairwiseNs", "empcovs", "rawData")) {
+      if (i %in% c("pairwiseNs", "empcovs", "rawData", "deltas", "emprawList")) {
         newStudyList[[counter]] <- primaryStudies[[counter]][problemStudy]
       } else {
         newStudyList[[counter]] <- list(unlist(primaryStudies[[counter]][problemStudy], recursive=TRUE))
@@ -103,8 +103,17 @@ ctmaOptimizeInit <- function(primaryStudies=NULL,
   names(newStudyList) <- names(primaryStudies)
   newStudyList$n.studies <- 1
 
+  #
+  # adaptations for dealing with raw data
+  if (!(is.null(newStudyList$rawData[[1]]$studyNumbers))) {
+    newStudyList$rawData[[1]]$studyNumbers <- 1
+    newStudyList$studyNumbers <- 1 # otherwise raw data will not be loaded
+    newStudyList$deltas <- unlist(newStudyList$deltas)
+  }
+
   # parallel re-fitting of problem study
   allfits <- foreach::foreach(i=1:reFits) %dopar% {
+    #head(newStudyList$emprawList[[1]])
     fits <- ctmaInit(newStudyList, coresToUse = 1, n.latent=n.latent,
                      activeDirectory = activeDirectory,
                      checkSingleStudyResults=checkSingleStudyResults,
@@ -116,9 +125,12 @@ ctmaOptimizeInit <- function(primaryStudies=NULL,
   bestFit <- which(unlist(all_minus2ll) == min(unlist(all_minus2ll)))[1]; bestFit
   bestFit <- allfits[[bestFit]]
 
-  results <- list(bestFit=bestFit, all_minus2ll=all_minus2ll, summary=bestFit$summary)
+  results <- list(bestFit=bestFit, all_minus2ll=all_minus2ll, summary=bestFit$summary,
+                  resultsSummary=bestFit$studyFitList[[1]]$resultsSummary
+  )
   class(results) <- "CoTiMAFit"
 
   invisible(results)
 }
 
+
